package com.cht.tlcsl.webrtcsample;

import android.app.Activity;
import android.app.FragmentManager;
import android.app.FragmentTransaction;
import android.app.ProgressDialog;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Color;
import android.os.Bundle;
import android.util.DisplayMetrics;
import android.util.Log;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.ProgressBar;
import android.widget.TextView;
import android.widget.Toast;
import com.cht.tlcsl.dbl.DBLMessageListener;
import com.cht.tlcsl.dbl.DBLSurfaceViewRendererInterface;
import com.cht.tlcsl.dbl.DrawingFragment;
import com.cht.tlcsl.webrtclib.M2MFragment;
import com.cht.tlcsl.webrtclib.WRLFileTransferListener;
import com.cht.tlcsl.webrtclib.WRLMessageListener;
import com.github.angads25.filepicker.controller.DialogSelectionListener;
import com.github.angads25.filepicker.model.DialogConfigs;
import com.github.angads25.filepicker.model.DialogProperties;
import com.github.angads25.filepicker.view.FilePickerDialog;
import java.io.File;


public class CalleeActivity extends Activity implements WRLMessageListener, DBLMessageListener {
    private static final String TAG = "CalleeActivity";
    M2MFragment mM2MFragment = null;
    String mCallerId;
    String mCallerName;
    String userId;
    String userToken;
    String serverPreUrl;
    //String mCalleeSeq = "";
    int mDisplayWidthInt;
    int mDisplayHeightInt;
    private ProgressDialog mProgressDialog = null;
    DrawingFragment mDrawingFragment;
    String mDisplayMessageString;
    String mErrorMessageString;
    boolean mIsFirstCallCallee;
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        mIsFirstCallCallee = true;
        if (savedInstanceState == null) {
            Log.d(TAG, "savedInstanceState = null");
            Bundle extras = getIntent().getExtras();
            if(extras == null) {
                Log.d(TAG, "extras = null");
                mCallerId = null;
                mCallerName = null;
            } else {
                Log.d(TAG, "extras != null");
                mCallerId = extras.getString("CallerId");
                Log.d(TAG, "mCallerId=" + mCallerId);
                mCallerName = extras.getString("CallerName");
                Log.d(TAG, "mCallerName=" + mCallerName);
            }
        } else {
            Log.d(TAG, "savedInstanceState != null");
            mCallerId= (String) savedInstanceState.getSerializable("CallerId");
            mCallerName= (String) savedInstanceState.getSerializable("CallerName");
        }
        if (mCallerId == null){
            Intent intent = new Intent();
            intent.setClass(this, M2MMainActivity.class);
            this.startActivity(intent);
            finish();
            return;
        }
        SharedPreferences sharedPreferences = getSharedPreferences("com.cht.tlcsl.webrtcsample.preferences", MODE_PRIVATE);
        userId = sharedPreferences.getString("AsstARUserId", null);
        userToken = sharedPreferences.getString("AsstARToken", null);
        serverPreUrl = sharedPreferences.getString("AsstARServerUrl", null);
        if (userId == null){
            Intent intent = new Intent();
            intent.setClass(this, LoginActivity.class);
            this.startActivity(intent);
            finish();
            return;
        }

        requestWindowFeature(Window.FEATURE_NO_TITLE);
        getWindow().setFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN,
                WindowManager.LayoutParams.FLAG_FULLSCREEN);
        //Add by Alec 2017.09.18 設定畫面為 IMMERSIVE_STICKY mode
        getWindow().getDecorView().setSystemUiVisibility(
                View.SYSTEM_UI_FLAG_LAYOUT_STABLE
                        | View.SYSTEM_UI_FLAG_LAYOUT_HIDE_NAVIGATION
                        | View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN
                        | View.SYSTEM_UI_FLAG_HIDE_NAVIGATION
                        | View.SYSTEM_UI_FLAG_FULLSCREEN
                        | View.SYSTEM_UI_FLAG_IMMERSIVE_STICKY);
        setContentView(R.layout.activity_callee);
        DisplayMetrics displayMetrics = new DisplayMetrics();
        getWindowManager().getDefaultDisplay().getMetrics(displayMetrics);
        mDisplayWidthInt = displayMetrics.widthPixels;
        mDisplayHeightInt = displayMetrics.heightPixels;
        int videoHeightInt = (int) (mDisplayWidthInt * 1280 / 720);
        if (mDisplayHeightInt > videoHeightInt)
            mDisplayHeightInt = videoHeightInt;
        //加入fragment
        mM2MFragment = M2MFragment.newInstance(userId, userToken, serverPreUrl);
        //設定是否顯示Debug訊息
        //設定Local Video Size
        //setLocalVideoSize(int inWidth, int inHeight, boolean isPortrait);
        //預設值為 720 * 1280 直式
        //BT-200  isProtrait設false為橫式
        //mM2MFragment.setLocalVideoSize(1280, 720, false);
        //Edit by Alec 2017.08.30 設定是否開啟Local Camera
        mM2MFragment.setEnableLocalCamera(false);
        //設定雙向視訊建立連線之timeout,預設值為 60000 , 60秒
        mM2MFragment.setWebrtcConnectTimeout(60000);
        mM2MFragment.setDebug(true);
        //mM2MFragment.setDebug(false);
        try {
            getFragmentManager().beginTransaction()
                    .replace(R.id.WRLM2MFragment, mM2MFragment)
                    .commit();
        } catch (RuntimeException e) {
            Toast.makeText(CalleeActivity.this, e.toString(), Toast.LENGTH_SHORT).show();
            finish();
            return;
        }
        //註冊檔案傳輸之Listener
        mM2MFragment.registerFileTransferListener(new CalleeActivity.FileTransferListener());
        //按Hangup時，呼叫M2MFragment的hangup Method
        Button hangupButton = (Button) findViewById(R.id.hangupButton);
        hangupButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {

                if (mDrawingFragment != null){
                    Log.d(TAG, "remove(mDrawingFragment)");
                    //將畫板Fragment從Activity刪除
                    try {
                        FragmentManager fragmentManager = getFragmentManager();
                        FragmentTransaction fragmentTransaction = fragmentManager.beginTransaction();
                        fragmentTransaction.remove(mDrawingFragment);
                        fragmentTransaction.commit();
                        mDrawingFragment = null;
                    } catch (Exception e){
                        Log.e(TAG, "remove(mDrawingFragment) exception=" + e.toString());
                        mDrawingFragment = null;
                    }
                }
                //mProgressDialog = ProgressDialog.show(CalleeActivity.this, "", "請稍候...", true, false);
                //掛斷
                mM2MFragment.hangup();
            }
        });
    }

    @Override
    public void onReceiveWRLMessage(int MessageType, String MessageString) {
        //MessageType=1 MessageString為 Text Message
        //MessageType=2 為白板image file，MessageString為白板image file所在位置
        //MessageType=3 為影音檔，MessageString為影音檔所在位置
        //MessageType=4 為凍結image file，MessageString為凍結image file所在位置
        //MessageType=5 為白板清除Message，MessageString固定為CLEAR BOARD
        //MessageType=6 為解凍Message，MessageString固定為UNFREEZE
        //MessageType=7 為其它檔，MessageString為其它檔所在位置
        Log.d(TAG, "onReceiveWRLMessage() type=" + MessageType + ";message=" + MessageString);
        mDisplayMessageString = "訊息 type=" + MessageType + ";message=" + MessageString;
        this.runOnUiThread(new Runnable() {
            @Override
            public void run() {
                Toast.makeText(CalleeActivity.this, mDisplayMessageString, Toast.LENGTH_SHORT).show();
            }
        });
        switch (MessageType){
            case 1:
                break;
            case 2:
                File boardImageFile = new  File(MessageString);
                Bitmap boardBitmap = BitmapFactory.decodeFile(boardImageFile.getAbsolutePath());
                ImageView boradImageView = (ImageView) findViewById(R.id.boardImageView);
                //Matrix matrix = new Matrix();
                //matrix.setScale((float)mDisplayWidthInt/(float)boardBitmap.getWidth(), (float)mDisplayWidthInt/(float)boardBitmap.getWidth(), 0, 0);
                //boradImageView.setImageMatrix(matrix);
                boradImageView.setVisibility(View.VISIBLE);
                boradImageView.setImageBitmap(boardBitmap);
                break;
            case 3:
                break;
            case 4:
                File freezeImageFile = new File(MessageString);
                Bitmap freezeBitmap = BitmapFactory.decodeFile(freezeImageFile.getAbsolutePath());
                ImageView freezeImageView = (ImageView) findViewById(R.id.freezeImageView);
                //Matrix freezeMatrix = new Matrix();
                //freezeMatrix.setScale((float)mDisplayWidthInt/(float)freezeBitmap.getWidth(), (float)mDisplayWidthInt/(float)freezeBitmap.getWidth(), 0, 0);
                //freezeImageView.setImageMatrix(freezeMatrix);
                freezeImageView.setVisibility(View.VISIBLE);
                freezeImageView.setImageBitmap(freezeBitmap);
                if (mDrawingFragment != null)
                    mDrawingFragment.setFreeze();
                break;
            case 5:
                boradImageView = (ImageView) findViewById(R.id.boardImageView);
                boradImageView.setVisibility(View.GONE);
                if (mDrawingFragment != null)
                    mDrawingFragment.clearDrawingBoard();
                break;
            case 6:
                freezeImageView = (ImageView) findViewById(R.id.freezeImageView);
                freezeImageView.setVisibility(View.GONE);
                if (mDrawingFragment != null)
                    mDrawingFragment.setUnfreeze();
                break;
            case 7:
            default:
                break;
        }
    }

    @Override
    public void onConnect(String inCalleeSeq) {
        //例如可在此改變Call Button由不可按變為可掛斷
        //mHandler.sendEmptyMessage(FINSH_CHANGE_FRAGMENT_PROGRESS_THREAD);
        Log.d(TAG, "onConnect() inCalleeSeq = " + inCalleeSeq);
        //mCalleeSeq = inCalleeSeq;
        this.runOnUiThread(new Runnable() {
            @Override
            public void run() {
                Toast.makeText(CalleeActivity.this, "onConnect()", Toast.LENGTH_SHORT).show();
                //隱藏支援端畫板畫面
                ImageView boradImageView = (ImageView) findViewById(R.id.boardImageView);
                boradImageView.setVisibility(View.GONE);
                //隱藏凍結畫面
                ImageView freezeImageView = (ImageView) findViewById(R.id.freezeImageView);
                freezeImageView.setVisibility(View.GONE);
                //顯示客服編號
                TextView calleeTextView = (TextView) findViewById(R.id.calleeTextView);
                //calleeTextView.setText("專家:" + mCalleeSeq);

                //將畫板Fragment加入Activity
                FragmentManager fragmentManager = getFragmentManager();
                FragmentTransaction fragmentTransaction = fragmentManager.beginTransaction();
                mDrawingFragment = DrawingFragment.newInstance();
                mDrawingFragment.setInitColor(Color.rgb(0, 255, 0));
                //移除LED按鈕
                //mDrawingFragment.removeLEDButton();
                fragmentTransaction.replace(R.id.drawingFrameLayout, mDrawingFragment, "DrawingFragment");
                fragmentTransaction.commit();
                mDrawingFragment.setDBLSurfaceViewRendererInterface((DBLSurfaceViewRendererInterface) findViewById(R.id.remote_video_view));
            }
        });
    }

    @Override
    public void onDisconnect() {
        //例如可在此改變Call Button變為可撥打
        //mHandler.sendEmptyMessage(FINSH_CHANGE_FRAGMENT_PROGRESS_THREAD);
        Log.d(TAG, "onDisconnect()");
        this.runOnUiThread(new Runnable() {
            @Override
            public void run() {
                //隱藏支援端畫板畫面
                ImageView boradImageView = (ImageView) findViewById(R.id.boardImageView);
                boradImageView.setVisibility(View.GONE);
                //隱藏凍結畫面
                ImageView freezeImageView = (ImageView) findViewById(R.id.freezeImageView);
                freezeImageView.setVisibility(View.GONE);
                //FragmentManager fragmentManager = getFragmentManager();
                //DrawingFragment drawingFragment = (DrawingFragment) fragmentManager.findFragmentByTag("DrawingFragment");
                if (mDrawingFragment != null){
                    Log.d(TAG, "remove(mDrawingFragment)");
                    //將畫板Fragment從Activity刪除
                    try {
                        FragmentManager fragmentManager = getFragmentManager();
                        FragmentTransaction fragmentTransaction = fragmentManager.beginTransaction();
                        fragmentTransaction.remove(mDrawingFragment);
                        fragmentTransaction.commit();
                        mDrawingFragment = null;
                    } catch (Exception e){
                        Log.e(TAG, "remove(mDrawingFragment) exception=" + e.toString());
                        mDrawingFragment = null;
                    }
                }
                Toast.makeText(CalleeActivity.this, "onDisconnect()", Toast.LENGTH_SHORT).show();
                finish();
            }
        });
    }

    @Override
    public void onError(int errorType, String descriptionString) {
        //ErrorType=11 Failed to create peer connection
        //ErrorType=12 Failed to open camera
        //ErrorType=13 Failed to add stream
        //ErrorType=14 Failed to create SDP
        //ErrorType=15 Failed to create ice connection
        //ErrorType=16 Failed to set SDP
        //ErrorType=17 Failed to initializeAndroidGlobals
        //ErrorType=21 Failed to connect WebSocket
        //ErrorType=22 Failed to register WebSocket
        //ErrorType=23 Failed to send WebSocket
        //ErrorType=24 Failed to send PostServer
        //ErrorType=25 無法連接訊息伺服器
        //ErrorType=26 與遠端失去連線
        //ErrorType=31 Failed to parse RoomParameters
        //ErrorType=32 onWebSocketError
        //ErrorType=33 not initiator receive answer
        //ErrorType=34  initiator receive offer
        //ErrorType=35 Unexpected WebSocket message
        //ErrorType=36 WebSocket message JSON parsing error
        //ErrorType=37 WebSocket non connected
        //ErrorType=38 WebSocket send error
        //ErrorType=39 Loopback room is busy
        //ErrorType=101 Permissions not granted
        //ErrorType=103 Failed to connect ARWORK API
        //ErrorType=104 Failed to parse ARWORK API JSON
        //ErrorType=105 使用者取消連線
        //ErrorType=107 對方取消連線
        //ErrorType=108 對方忙線中
        //ErrorType=109 重覆呼叫SDK
        //mHandler.sendEmptyMessage(FINSH_CHANGE_FRAGMENT_PROGRESS_THREAD);
        Log.d(TAG, "onError() type=" + errorType + ";description=" + descriptionString);
        mErrorMessageString = "訊息 type=" + errorType + ";description=" + descriptionString;
        this.runOnUiThread(new Runnable() {
            @Override
            public void run() {
                //隱藏支援端畫板畫面
                ImageView boradImageView = (ImageView) findViewById(R.id.boardImageView);
                boradImageView.setVisibility(View.GONE);
                //隱藏凍結畫面
                ImageView freezeImageView = (ImageView) findViewById(R.id.freezeImageView);
                freezeImageView.setVisibility(View.GONE);
                //FragmentManager fragmentManager = getFragmentManager();
                //DrawingFragment drawingFragment = (DrawingFragment) fragmentManager.findFragmentByTag("DrawingFragment");
                if (mDrawingFragment != null){
                    Log.d(TAG, "remove(mDrawingFragment)");
                    //將畫板Fragment從Activity刪除
                    try {
                        FragmentManager fragmentManager = getFragmentManager();
                        FragmentTransaction fragmentTransaction = fragmentManager.beginTransaction();
                        fragmentTransaction.remove(mDrawingFragment);
                        fragmentTransaction.commit();
                        mDrawingFragment = null;
                    } catch (Exception e){
                        Log.e(TAG, "remove(mDrawingFragment) exception=" + e.toString());
                        mDrawingFragment = null;
                    }
                }
                Toast.makeText(CalleeActivity.this, mErrorMessageString, Toast.LENGTH_SHORT).show();
            }
        });
        //finish();
    }
    //手機端畫板畫面已儲存完畢之Callback
    @Override
    public void onDBLFileSaved(String filePathString){
        Log.d(TAG, "onDBLFileSaved()");
        //傳送畫板畫面檔案至支援端
        if (mM2MFragment != null)
            mM2MFragment.sendLocalBoardFile(filePathString);
        //Toast.makeText(this, "onDBLFileSaved():" + filePathString, Toast.LENGTH_SHORT).show();
    }
    //手機端使用者按下清除畫板之Callback
    @Override
    public void onDBLClear(){
        Log.d(TAG, "onDBLClear()");
        //清除支援端畫板
        ImageView boradImageView = (ImageView) findViewById(R.id.boardImageView);
        boradImageView = (ImageView) findViewById(R.id.boardImageView);
        boradImageView.setVisibility(View.GONE);
        //傳送清除畫板message至支援端
        if (mM2MFragment != null)
            mM2MFragment.sendClearingBoardRequest();
        //Toast.makeText(this, "onDBLClear()", Toast.LENGTH_SHORT).show();
    }
    //手機端凍結畫面已儲存完畢之Callback
    @Override
    public void onDBLFreezeFileSaved(String filePathString){
        Log.d(TAG, "onDBLFreezeFileSaved()");
        //顯示手機端凍結畫面
        File freezeImageFile = new File(filePathString);
        Bitmap freezeBitmap = BitmapFactory.decodeFile(freezeImageFile.getAbsolutePath());
        ImageView freezeImageView = (ImageView) findViewById(R.id.freezeImageView);
        freezeImageView.setVisibility(View.VISIBLE);
        freezeImageView.setImageBitmap(freezeBitmap);
        //傳送凍結畫面檔案至支援端
        if (mM2MFragment != null)
            mM2MFragment.sendLocalFreezingFile(filePathString);
        //Toast.makeText(CalleeActivity.this, "onDBLFreezeFileSaved()", Toast.LENGTH_SHORT).show();
    }
    //手機端使用者按下解凍按鈕之Callback
    @Override
    public void onDBLUnfreeze(){
        Log.d(TAG, "onDBLUnfreeze()");
        //清除手機端凍結畫面
        ImageView freezeImageView = (ImageView) findViewById(R.id.freezeImageView);
        freezeImageView.setVisibility(View.GONE);
        //傳送解凍message至支援端
        if (mM2MFragment != null)
            mM2MFragment.sendUnfreezingBoardRequest();
        //Toast.makeText(CalleeActivity.this, "onDBLUnfreeze()", Toast.LENGTH_SHORT).show();
    }
    //手機端使用者按下文字輸入按鈕之Callback
    int mInputCount = 0;
    @Override
    public void onDBLInputButtonPressed(){
        Log.d(TAG, "onDBLInputButtonPressed()");
        //在此顯示使用者文字輸入畫面，輸入完成以下方式傳給對方
        if (mM2MFragment != null)
            mM2MFragment.sendTextMessage("測試Message" + mInputCount++);
    }
    @Override
    //手機端使用者按下傳送檔案按鈕之Callback
    public void onDBLFileButtonPressed(){
        Log.d(TAG, "onDBLFileButtonPressed()");
        //在此顯示使用者選取檔案畫面，輸入完成以下方式傳給對方
        if (mM2MFragment != null){
            DialogProperties properties = new DialogProperties();
            properties.selection_mode = DialogConfigs.SINGLE_MODE;
            properties.selection_type = DialogConfigs.FILE_SELECT;
            properties.root = new File(DialogConfigs.DEFAULT_DIR);
            properties.error_dir = new File(DialogConfigs.DEFAULT_DIR);
            properties.offset = new File(DialogConfigs.DEFAULT_DIR);
            properties.extensions = null;
            FilePickerDialog dialog = new FilePickerDialog(CalleeActivity.this,properties);
            dialog.setTitle("Select a File");
            dialog.setDialogSelectionListener(new DialogSelectionListener() {
                @Override
                public void onSelectedFilePaths(String[] files) {
                    if (files.length > 0){
                        Log.d(TAG, "onSelectedFilePaths() files[0]=" + files[0]);
                        //mM2MFragment.sendLocalFile(files[0]);
                        mM2MFragment.noBlockSendLocalFile(files[0]);
                    }
                }
            });
            dialog.show();
        }
    }
    @Override
    //手機端使用者按下LED按鈕之Callback
    public void onDBLLedButtonPressed(){
        Log.d(TAG, "onDBLLedButtonPressed()");
        //按LED時，呼叫M2MFragment的onOffFlashLight Method
        if (mM2MFragment != null)
            mM2MFragment.onOffFlashLight();
    }
    //@Override
    //public void onBackPressed() {
        // do nothing.
    //}
    @Override
    public void onResume() {
        super.onResume();
        Log.d(TAG, "onResume()");
        if (mM2MFragment != null){
            if (mIsFirstCallCallee){
                mIsFirstCallCallee = false;
                mM2MFragment.callee(mCallerId, R.id.local_video_view, R.id.remote_video_view, true);
            }
        }
    }
    @Override
    public void onPause() {
        super.onPause();
        Log.d(TAG, "onPause()");
    }
    String mSendFileName;
    long mSendFileSize;
    int mSendMaxValue;
    int mSendMinValue;
    int mSendProgressValue;
    String mReceiveFileName;
    long mReceiveFileSize;
    int mReceiveMaxValue;
    int mReceiveMinValue;
    int mReceiveProgressValue;
    private class FileTransferListener implements WRLFileTransferListener {

        @Override
        public void onSendProgressUpdate(String fileName , final long fileSize, int maxValue, int minValue, int progressValue) {
            //fileName:欲傳送檔案短檔名  fileSize:檔案大小 maxValue:Progress的最大值 minValue:Progress的最小值 progressVaule:目前Progress之值
            Log.d(TAG, "onSendProgressUpdate() fileName=" + fileName + ";maxValue=" + maxValue + ";minValue=" + minValue + ";progressValue=" + progressValue);
            mSendFileName = fileName;
            mSendFileSize = fileSize;
            mSendMaxValue = maxValue;
            mSendMinValue = minValue;
            mSendProgressValue = progressValue;
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    TextView sendFileNameTextView = (TextView) findViewById(R.id.sendFileNameTextView);
                    sendFileNameTextView.setVisibility(View.VISIBLE);
                    sendFileNameTextView.setText("傳送:" + mSendFileName + " " + (mSendFileSize / 1048576) + "M");
                    ProgressBar sendFileProgressBar = (ProgressBar) findViewById(R.id.sendFileProgressBar);
                    sendFileProgressBar.setVisibility(View.VISIBLE);
                    sendFileProgressBar.setMax(mSendMaxValue);
                    sendFileProgressBar.setProgress(mSendProgressValue);
                }
            });
        }

        @Override
        public void onSendFinish(String fileName) {
            //完成傳送檔案 fileName:欲傳送檔案短檔名
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    TextView sendFileNameTextView = (TextView) findViewById(R.id.sendFileNameTextView);
                    sendFileNameTextView.setVisibility(View.GONE);
                    ProgressBar sendFileProgressBar = (ProgressBar) findViewById(R.id.sendFileProgressBar);
                    sendFileProgressBar.setVisibility(View.GONE);
                }
            });
        }

        @Override
        public void onReceiveProgressUpdate(String fileName, long fileSize, int maxValue, int minValue, int progressValue) {
            //fileName:接收檔案短檔名 fileSize:檔案大小 maxValue:Progress的最大值 minValue:Progress的最小值 progressVaule:目前Progress之值
            //舊版接收檔案因無檔案大小，所以 fileSize,maxValue,minValue,progressValue皆為0
            Log.d(TAG, "onReceiveProgressUpdate() fileName=" + fileName + ";maxValue=" + maxValue + ";minValue=" + minValue + ";progressValue=" + progressValue);
            mReceiveFileName = fileName;
            mReceiveFileSize = fileSize;
            mReceiveMaxValue = maxValue;
            mReceiveMinValue = minValue;
            mReceiveProgressValue = progressValue;
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    if (mReceiveFileSize == 0){
                        //舊版接收檔案因無檔案大小，所以 fileSize,maxValue,minValue,progressValue皆為0
                        TextView receiveFileNameTextView = (TextView) findViewById(R.id.receiveFileNameTextView);
                        receiveFileNameTextView.setVisibility(View.VISIBLE);
                        receiveFileNameTextView.setText("接收:" + mReceiveFileName);
                        ProgressBar receiveFileProgressBar = (ProgressBar) findViewById(R.id.receiveFileProgressBar);
                        receiveFileProgressBar.setVisibility(View.VISIBLE);
                        receiveFileProgressBar.setMax(100);
                        receiveFileProgressBar.setProgress(100);
                    } else {
                        TextView receiveFileNameTextView = (TextView) findViewById(R.id.receiveFileNameTextView);
                        receiveFileNameTextView.setVisibility(View.VISIBLE);
                        receiveFileNameTextView.setText("接收:" + mReceiveFileName + " " + (mReceiveFileSize / 1048576) + "M");
                        ProgressBar receiveFileProgressBar = (ProgressBar) findViewById(R.id.receiveFileProgressBar);
                        receiveFileProgressBar.setVisibility(View.VISIBLE);
                        receiveFileProgressBar.setMax(mReceiveMaxValue);
                        receiveFileProgressBar.setProgress(mReceiveProgressValue);
                    }
                }
            });
        }

        @Override
        public void onReceiveFinish(String fileName, String fullFileName) {
            //完成接收檔案 fileName:接收檔案短檔名 fullFileName:接收檔案完整的儲存檔名
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    TextView receiveFileNameTextView = (TextView) findViewById(R.id.receiveFileNameTextView);
                    receiveFileNameTextView.setVisibility(View.GONE);
                    ProgressBar receiveFileProgressBar = (ProgressBar) findViewById(R.id.receiveFileProgressBar);
                    receiveFileProgressBar.setVisibility(View.GONE);
                }
            });
        }
    }
    @Override
    protected void onDestroy() {
        Log.d(TAG, "onDestroy()");
        if (mM2MFragment != null)
            mM2MFragment.unRegisterFileTransferListener();
        super.onDestroy();
    }
}
