package com.cht.tlcsl.webrtcsample;

import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;
import android.app.Fragment;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.util.Log;

/**
 * A fragment representing a list of Items.
 * <p/>
 */
public class CalleesListFragment extends Fragment {

    private static final String LOG_TAG = "CalleesListFragment";

    private static final String ARG_CONTENT_ARRAY = "content-array";
    private String[] mContentArray;

    /**
     * Mandatory empty constructor for the fragment manager to instantiate the
     * fragment (e.g. upon screen orientation changes).
     */
    public CalleesListFragment() {
    }

    // TODO: Customize parameter initialization
    @SuppressWarnings("unused")
    public static CalleesListFragment newInstance(String[] inContentArray) {
        CalleesListFragment fragment = new CalleesListFragment();
        Bundle args = new Bundle();
        args.putStringArray(ARG_CONTENT_ARRAY, inContentArray);
        fragment.setArguments(args);
        return fragment;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        if (getArguments() != null) {
            mContentArray = getArguments().getStringArray(ARG_CONTENT_ARRAY);
        }
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.fragment_callees_list, container, false);

        // Set the adapter
        if (view instanceof RecyclerView) {
            Context context = view.getContext();
            RecyclerView recyclerView = (RecyclerView) view;
            recyclerView.setNestedScrollingEnabled(false);
            recyclerView.setHasFixedSize(false);
            recyclerView.setLayoutManager(new LinearLayoutManager(context));
            recyclerView.setAdapter(new CalleesListItemRecyclerViewAdapter(mContentArray));
        }
        return view;
    }


    @Override
    public void onAttach(Context context) {
        super.onAttach(context);
    }

    @Override
    public void onDetach() {
        super.onDetach();
    }

    /**
     * {@link RecyclerView.Adapter} that can display a {@link DummyItem} and makes a call to the
     * TODO: Replace the implementation with code for your data type.
     */
    public class CalleesListItemRecyclerViewAdapter extends RecyclerView.Adapter<CalleesListItemRecyclerViewAdapter.ViewHolder> {

        private String[] mContentArray;

        public CalleesListItemRecyclerViewAdapter(String[] inContentArray) {
            mContentArray = inContentArray;
        }

        @Override
        public ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
            View view = LayoutInflater.from(parent.getContext())
                    .inflate(R.layout.item_callees_list, parent, false);
            return new ViewHolder(view);
        }

        @Override
        public void onBindViewHolder(final ViewHolder holder, final int position) {
            holder.mContentView.setText(mContentArray[position]);

            holder.mView.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    Log.d(LOG_TAG, "onClick position=" + position);
                    ((MainActivity)getActivity()).onCalleesListFragmentClick(position);
                }
            });
        }

        @Override
        public int getItemCount() {
            return mContentArray.length;
        }

        public class ViewHolder extends RecyclerView.ViewHolder {
            public final View mView;
            public final TextView mContentView;

            public ViewHolder(View view) {
                super(view);
                mView = view;
                mContentView = (TextView) view.findViewById(R.id.content);
            }

            @Override
            public String toString() {
                return super.toString() + " '" + mContentView.getText() + "'";
            }
        }
    }
}
