package com.cht.tlcsl.webrtcsample;

import android.Manifest;
import android.animation.Animator;
import android.animation.AnimatorListenerAdapter;
import android.annotation.TargetApi;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.AsyncTask;
import android.os.Build;
import android.os.Message;
import android.os.RemoteException;
import android.support.v4.content.ContextCompat;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Toast;
import android.util.Log;

import com.cht.tlcsl.webrtclib.M2MFragment;
import com.cht.tlcsl.webrtclib.WRLSignalService;

import org.json.JSONObject;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.HttpURLConnection;
import java.net.URL;

/**
 * A login screen
 */
public class LoginActivity extends AppCompatActivity {

    private final static String LOG_TAG = "LoginActivity";
    private static final boolean VERBOSE = true;
    private static final int READ_TIMEOUT = 20 * 1000;
    private boolean activityRunning;
    private static final String[] MANDATORY_PERMISSIONS = {
            "android.permission.CAMERA",
            "android.permission.WRITE_EXTERNAL_STORAGE",
            "android.permission.ACCESS_NETWORK_STATE",
            "android.permission.MODIFY_AUDIO_SETTINGS",
            "android.permission.RECORD_AUDIO",
            "android.permission.INTERNET"
    };
    private static final int PERMISSIONS_REQUEST = 3001;

    /**
     * Keep track of the login task to ensure we can cancel it if requested.
     */
    private UserLoginTask mAuthTask = null;

    // UI references.
    private EditText mServerPreURLEditText;
    private EditText mNickNameEditText;
    private EditText mVerifyCodeEditText;
    private View mProgressView;
    private View mLoginView;
    private String mServerPreURL;
    private String mServerURL;
    private String mUserToken;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        M2MFragment.stopWRLSignalService(getApplicationContext());
        setContentView(R.layout.activity_login);
        // Set up the login form.
        SharedPreferences sharedPreferences = getSharedPreferences("com.cht.tlcsl.webrtcsample.preferences", MODE_PRIVATE);
        mServerPreURLEditText = (EditText) findViewById(R.id.serverURLEditText);
        mServerPreURLEditText.setText(sharedPreferences.getString("ServerPreURL", "https://asstar-qa.hinet.net/AsstARWeb/"));
        mNickNameEditText = (EditText) findViewById(R.id.nickNameEditText);
        mNickNameEditText.setText(sharedPreferences.getString("UserId", "cht007"));
        mNickNameEditText.requestFocus();

        mVerifyCodeEditText = (EditText) findViewById(R.id.verifyCodeEditText);
        mVerifyCodeEditText.setText(sharedPreferences.getString("UserPw", "cht007"));

        Button mLoginButton = (Button) findViewById(R.id.loginButton);
        mLoginButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                attemptLogin();
            }
        });

        mLoginView = findViewById(R.id.loginScrollView);
        mProgressView = findViewById(R.id.loginProgressBar);
        //check permission
        if (Build.VERSION.SDK_INT >= 23){
            for (String permission : MANDATORY_PERMISSIONS) {
                if (this.checkCallingOrSelfPermission(permission) != PackageManager.PERMISSION_GRANTED) {
                    if (VERBOSE) Log.d(LOG_TAG, "no permission");
                    requestPermissions(MANDATORY_PERMISSIONS,
                            PERMISSIONS_REQUEST);
                }
            }
        }
    }

    /**
     * Attempts to sign in or register the account specified by the login form.
     * If there are form errors (invalid email, missing fields, etc.), the
     * errors are presented and no actual login attempt is made.
     */
    private void attemptLogin() {

        if (mAuthTask != null) {
            return;
        }

        // Reset errors.
        mNickNameEditText.setError(null);
        mVerifyCodeEditText.setError(null);

        // Store values at the time of the login attempt.
        SharedPreferences sharedPreferences = getSharedPreferences("com.cht.tlcsl.webrtcsample.preferences", MODE_PRIVATE);
        SharedPreferences.Editor editor = sharedPreferences.edit();
        editor.putString("ServerPreURL", mServerPreURLEditText.getText().toString());
        editor.putString("UserId", mNickNameEditText.getText().toString());
        editor.putString("UserPw", mVerifyCodeEditText.getText().toString());
        editor.commit();
        mServerPreURL = mServerPreURLEditText.getText().toString();
        mServerURL = mServerPreURL + "webservice/userlogin";

        String nickNameString = mNickNameEditText.getText().toString();
        String verifyCodeString = mVerifyCodeEditText.getText().toString();

        boolean cancel = false;
        View focusView = mNickNameEditText;

        Log.d(LOG_TAG, "before isNickNameValid");
        // Check for a valid  nickName.
        if (TextUtils.isEmpty(nickNameString)) {
            mNickNameEditText.setError("請輸入帳號");
            focusView = mNickNameEditText;
            cancel = true;
            Log.d(LOG_TAG, "NickName invalid");
        } else if (!isNickNameValid(nickNameString)) {
            mNickNameEditText.setError("請輸入正確帳號");
            focusView = mNickNameEditText;
            cancel = true;
            Log.d(LOG_TAG, "NickName invalid");
        }

        Log.d(LOG_TAG, "before isVerifyCodeValid");
        // Check for a valid VerifyCode, if the user entered one.
        if (TextUtils.isEmpty(verifyCodeString)) {
            mVerifyCodeEditText.setError("請輸入密碼");
            focusView = mVerifyCodeEditText;
            cancel = true;
            Log.d(LOG_TAG, "VerifyCode invalid");
        } else if (!isVerifyCodeValid(verifyCodeString)) {
            mVerifyCodeEditText.setError("請輸入正確密碼");
            focusView = mVerifyCodeEditText;
            cancel = true;
            Log.d(LOG_TAG, "VerifyCode invalid");
        }

        if (cancel) {
            // There was an error; don't attempt login and focus the first
            // form field with an error.
            focusView.requestFocus();
        } else {
            // Show a progress spinner, and kick off a background task to
            // perform the user login attempt.
            String requestUrlString = mServerURL;
            //String requestContentString = "op=Validation&identifier=" + this.getPackageName() + "&id=" + nickNameString + "&password=" + verifyCodeString;
            String requestContentString = "op=Validation&identifier=" + "com.cht.assistantar.qa" + "&id=" + nickNameString + "&password=" + verifyCodeString;
            showProgress(true);
            mAuthTask = new UserLoginTask(requestUrlString, requestContentString);
            mAuthTask.execute();
        }
    }
    private boolean isNickNameValid(String nickNameString) {
        //TODO: Replace this with your own logic
        Log.d(LOG_TAG, "nickName=" + nickNameString);
        return nickNameString.length() > 2;
    }

    private boolean isVerifyCodeValid(String verifyCodeString) {
        //TODO: Replace this with your own logic
        return verifyCodeString.length() > 2;
    }

    /**
     * Shows the progress UI and hides the login form.
     */
    @TargetApi(Build.VERSION_CODES.HONEYCOMB_MR2)
    private void showProgress(final boolean show) {
        // On Honeycomb MR2 we have the ViewPropertyAnimator APIs, which allow
        // for very easy animations. If available, use these APIs to fade-in
        // the progress spinner.
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.HONEYCOMB_MR2) {
            int shortAnimTime = getResources().getInteger(android.R.integer.config_shortAnimTime);

            mLoginView.setVisibility(show ? View.GONE : View.VISIBLE);
            mLoginView.animate().setDuration(shortAnimTime).alpha(
                    show ? 0 : 1).setListener(new AnimatorListenerAdapter() {
                @Override
                public void onAnimationEnd(Animator animation) {
                    mLoginView.setVisibility(show ? View.GONE : View.VISIBLE);
                }
            });

            mProgressView.setVisibility(show ? View.VISIBLE : View.GONE);
            mProgressView.animate().setDuration(shortAnimTime).alpha(
                    show ? 1 : 0).setListener(new AnimatorListenerAdapter() {
                @Override
                public void onAnimationEnd(Animator animation) {
                    mProgressView.setVisibility(show ? View.VISIBLE : View.GONE);
                }
            });
        } else {
            // The ViewPropertyAnimator APIs are not available, so simply show
            // and hide the relevant UI components.
            mProgressView.setVisibility(show ? View.VISIBLE : View.GONE);
            mLoginView.setVisibility(show ? View.GONE : View.VISIBLE);
        }
    }

    /**
     * Represents an asynchronous login/registration task used to authenticate
     * the user.
     */
    public class UserLoginTask extends AsyncTask<Void, Void, JSONObject> {

        private final String mRequestUrlString;
        private final String mRequestContentString;

        UserLoginTask(String requestUrlString, String requestContentString) {
            mRequestUrlString = requestUrlString;
            mRequestContentString = requestContentString;
        }

        @Override
        protected JSONObject doInBackground(Void... params) {
            return requestARCOAPI(mRequestUrlString, mRequestContentString);
        }

        @Override
        protected void onPostExecute(JSONObject resultJSONObject) {
            mAuthTask = null;
            showProgress(false);
            try {
                String stat_codeString = resultJSONObject.getString("status_code");
                if (stat_codeString.equalsIgnoreCase("1000")){
                    mUserToken = resultJSONObject.getString("session_id");
                    goToMainActivity();
                } else {
                    if (activityRunning){
                        new AlertDialog.Builder(LoginActivity.this).setTitle("失敗").setMessage(resultJSONObject.getString("status_msg"))
                                .setPositiveButton("確定", new DialogInterface.OnClickListener() {
                                    public void onClick(DialogInterface dialog, int which) {
                                        mVerifyCodeEditText.requestFocus();
                                    }
                                }).setCancelable(false).create().show();
                    }
                    return;
                }
            } catch (Exception e){
                Log.d(LOG_TAG, "onPostExecute() JSON parse error=" + e.toString());
                if (activityRunning) {
                    new AlertDialog.Builder(LoginActivity.this).setTitle("失敗").setMessage("請稍候再試!")
                            .setPositiveButton("確定", new DialogInterface.OnClickListener() {
                                public void onClick(DialogInterface dialog, int which) {
                                }
                            }).setCancelable(false).create().show();
                }
            }
        }

        @Override
        protected void onCancelled() {
            mAuthTask = null;
            showProgress(false);
        }
    }
    private JSONObject requestARCOAPI(String requestUrlString, String requestContentString) {
        HttpURLConnection httpURLConnection = null;
        BufferedReader bufferedReader = null;
        InputStream inputStream = null;
        InputStreamReader inputStreamReader = null;
        PrintWriter contentPrintWriter = null;
        String charset = "UTF-8";

        try {

            if (VERBOSE) Log.d(LOG_TAG, "requestARCOAPI() requestUrlString=" + requestUrlString + ";requestContentString=" + requestContentString);
            System.setProperty("http.keepAlive", "false");

            URL url = new URL(requestUrlString);
            //URL url = new URL(requestUrlString + "?" + requestContentString);
            // Open a HTTPS  connection to  the URL
            httpURLConnection = (HttpURLConnection) url.openConnection();
            httpURLConnection.setConnectTimeout(READ_TIMEOUT * 2);
            httpURLConnection.setReadTimeout(READ_TIMEOUT);
            httpURLConnection.setRequestMethod("POST");
            //httpURLConnection.setRequestMethod("GET");
            httpURLConnection.setRequestProperty("Accept-Charset", charset);
            httpURLConnection.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");
            httpURLConnection.setRequestProperty("Content-Length", String.valueOf(requestContentString.length()));
            httpURLConnection.setUseCaches(false); // Don't use a Cached Copy
            httpURLConnection.setDoInput(true); // Allow Inputs
            httpURLConnection.setDoOutput(true); // Allow Outputs

            //httpURLConnection.setFixedLengthStreamingMode(requestContentString.getBytes().length);
            contentPrintWriter = new PrintWriter(httpURLConnection.getOutputStream());
            contentPrintWriter.print(requestContentString);
            contentPrintWriter.flush();
            contentPrintWriter.close();
            contentPrintWriter = null;

            // status code
            int httpStatusCode = httpURLConnection.getResponseCode();
            if (httpStatusCode != HttpURLConnection.HTTP_OK) {
                if (VERBOSE)
                    Log.e(LOG_TAG, String.format("[%s][%s]HTTP Status code : %d", this.getClass().getSimpleName(), Thread.currentThread().getStackTrace()[2].getMethodName(), httpStatusCode));
                // error response
                StringBuffer responseStringBuffer = new StringBuffer();
                String line = "";
                inputStream = httpURLConnection.getErrorStream();
                inputStreamReader = new InputStreamReader(inputStream, "UTF-8");
                bufferedReader = new BufferedReader(inputStreamReader);
                while ((line = bufferedReader.readLine()) != null) {
                    responseStringBuffer.append(line);
                }
                bufferedReader.close();
                bufferedReader = null;
                inputStreamReader.close();
                inputStreamReader = null;
                inputStream.close();
                inputStream = null;
                httpURLConnection.disconnect();
                httpURLConnection = null;
                String responseStr = responseStringBuffer.toString();
                if (VERBOSE)
                    Log.i(LOG_TAG, String.format("[%s][%s] 伺服端回傳結果 : %s ", this.getClass().getSimpleName(), Thread.currentThread().getStackTrace()[2].getMethodName(), responseStr));
                JSONObject jsonObject = new JSONObject();
                jsonObject.put("status_code", 12);
                jsonObject.put("status_msg", "連接伺服器失敗!");
                return jsonObject;
            }

            // response
            StringBuffer responseStringBuffer = new StringBuffer();
            String line = "";
            inputStream = httpURLConnection.getInputStream();
            inputStreamReader = new InputStreamReader(inputStream, "UTF-8");
            bufferedReader = new BufferedReader(inputStreamReader);
            while ((line = bufferedReader.readLine()) != null) {
                responseStringBuffer.append(line);
            }
            bufferedReader.close();
            bufferedReader = null;
            inputStreamReader.close();
            inputStreamReader = null;
            inputStream.close();
            inputStream = null;
            httpURLConnection.disconnect();
            httpURLConnection = null;
            String responseString = responseStringBuffer.toString();
            if (VERBOSE)
                Log.i(LOG_TAG, String.format("[%s][%s] 伺服端回傳結果 : %s ", this.getClass().getSimpleName(), Thread.currentThread().getStackTrace()[2].getMethodName(), responseString));
            try {
                JSONObject jsonObject = new JSONObject(responseString);
                return jsonObject;
            } catch (Exception e) {
                if (VERBOSE)
                    Log.e(LOG_TAG, "JSON parse error=" + e.toString());
                JSONObject jsonObject = new JSONObject();
                jsonObject.put("status_code", 13);
                jsonObject.put("status_msg", "資料解析失敗!");
                return jsonObject;
            }

        } catch (Exception ex) {
            if (VERBOSE)
                Log.e(LOG_TAG, String.format("[%s][%s]Exception : %s", this.getClass().getSimpleName(), Thread.currentThread().getStackTrace()[2].getMethodName(), ex.getMessage()));
            //ex.printStackTrace();
            JSONObject jsonObject = new JSONObject();
            try {
                jsonObject.put("status_code", 12);
                jsonObject.put("status_msg", "連接伺服器失敗!");
            } catch (Exception ex1) {
                if (VERBOSE)
                    Log.e(LOG_TAG, String.format("[%s][%s]Exception : %s", this.getClass().getSimpleName(), Thread.currentThread().getStackTrace()[2].getMethodName(), ex1.getMessage()));
            }
            return jsonObject;
        } finally {
            try {
                if (httpURLConnection != null)
                    httpURLConnection.disconnect();
            } catch (Exception ex) {
                if (VERBOSE)
                    Log.e(LOG_TAG, String.format("[%s][%s]Exception(in finally) : %s", this.getClass().getSimpleName(), Thread.currentThread().getStackTrace()[2].getMethodName(), ex.getMessage()));
            }
            try {
                if (inputStreamReader != null)
                    inputStreamReader.close();
            }catch (Exception ex){
                if (VERBOSE)
                    Log.e(LOG_TAG, String.format("[%s][%s]Exception(in finally) : %s", this.getClass().getSimpleName(), Thread.currentThread().getStackTrace()[2].getMethodName(), ex.getMessage()));
            }
            try {
                if (inputStream != null)
                    inputStream.close();
            }catch (Exception ex){
                if (VERBOSE)
                    Log.e(LOG_TAG, String.format("[%s][%s]Exception(in finally) : %s", this.getClass().getSimpleName(), Thread.currentThread().getStackTrace()[2].getMethodName(), ex.getMessage()));
            }
            try {
                if (bufferedReader != null)
                    bufferedReader.close();
            }catch (Exception ex){
                if (VERBOSE)
                    Log.e(LOG_TAG, String.format("[%s][%s]Exception(in finally) : %s", this.getClass().getSimpleName(), Thread.currentThread().getStackTrace()[2].getMethodName(), ex.getMessage()));
            }
            try {
                if (contentPrintWriter != null)
                    contentPrintWriter.close();
            }catch (Exception ex){
                if (VERBOSE)
                    Log.e(LOG_TAG, String.format("[%s][%s]Exception(in finally) : %s", this.getClass().getSimpleName(), Thread.currentThread().getStackTrace()[2].getMethodName(), ex.getMessage()));
            }
        }
    }
    private void goToMainActivity() {
        //啟動WRLSignalService
        //SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(LoginActivity.this);
        //SharedPreferences.Editor editor = sharedPreferences.edit();
        SharedPreferences sharedPreferences = getSharedPreferences("com.cht.tlcsl.webrtcsample.preferences", MODE_PRIVATE);
        SharedPreferences.Editor editor = sharedPreferences.edit();
        editor.putString("AsstARToken", mUserToken);
        editor.commit();
        Intent serviceIntent = new Intent(LoginActivity.this, WRLSignalService.class);
        serviceIntent.putExtra("UserId", mNickNameEditText.getText().toString());
        serviceIntent.putExtra("PreferencesName", "com.cht.tlcsl.webrtcsample.preferences");
        serviceIntent.putExtra("TokenName", "AsstARToken");
        serviceIntent.putExtra("LoginActionName", "com.cht.tlcsl.webrtcsample.LoginActivity");
        serviceIntent.putExtra("CalleeActionName", "com.cht.tlcsl.webrtcsample.PreCalleeActivity");
        serviceIntent.putExtra("ServerUrl", mServerPreURL);
        serviceIntent.putExtra("ConnectIcon", R.drawable.start);
        serviceIntent.putExtra("DisconnectIcon", R.drawable.stop);
        startService(serviceIntent);
        //啟動M2MMainActivity
        Intent intent = new Intent();
        Log.d(LOG_TAG, "userId=" + mNickNameEditText.getText().toString());
        intent.putExtra("userId", mNickNameEditText.getText().toString());
        Log.d(LOG_TAG, "mUserToken=" + mUserToken);
        intent.putExtra("userToken", mUserToken);
        Log.d(LOG_TAG, "mServerPreURL=" + mServerPreURL);
        intent.putExtra("serverPreUrl", mServerPreURL);
        //intent.setClass(this, MainActivity.class);
        //intent.setClass(this, CallerActivity.class);
        //intent.setClass(this, CalleeActivity.class);
        intent.setClass(this, M2MMainActivity.class);
        this.startActivity(intent);
        Toast.makeText(this, "驗證通過!", Toast.LENGTH_LONG).show();
        Log.d(LOG_TAG, "驗證通過");
        finish();
    }
    @Override
    public void onResume() {
        super.onResume();
        if (VERBOSE) Log.d(LOG_TAG, "onResume()");
        activityRunning = true;
        /*
        if (!isOpenNetwork() || !isOnline()){
            new AlertDialog.Builder(this).setTitle("錯誤").setMessage("請開啟網路!")
                    .setPositiveButton("確定", new DialogInterface.OnClickListener() {
                        public void onClick(DialogInterface dialog, int which) {
                            startActivity(new Intent(Settings.ACTION_SETTINGS));
                            finish();
                        }
                    }).setCancelable(false).create().show();
            return;
        }*/
        //Log.v(LOG_TAG, "onResume");
    }
    @Override
    public void onPause() {
        super.onPause();
        if (VERBOSE) Log.d(LOG_TAG, "onPause()");
        activityRunning = false;
    }
    @Override
    public void onRequestPermissionsResult(int requestCode,
                                           String permissions[], int[] grantResults) {
        switch (requestCode) {
            case PERMISSIONS_REQUEST:
                // If request is cancelled, the result arrays are empty.
                if (VERBOSE)
                    Log.d(LOG_TAG, "onRequestPermissionsResult()");
                if (VERBOSE)
                    Log.d(LOG_TAG, "grantResults.length=" + grantResults.length);
                if (grantResults.length == MANDATORY_PERMISSIONS.length){
                    for (int resultInt : grantResults) {
                        if (resultInt != PackageManager.PERMISSION_GRANTED){
                            Toast.makeText(this, "請開啟權限!", Toast.LENGTH_LONG).show();
                            finish();
                        }
                    }
                }
                break;
        }
    }
}

