package com.cht.tlcsl.webrtcsample;

import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;
import android.app.Fragment;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.util.Log;

/**
 * A fragment representing a list of Items.
 * <p/>
 */
public class M2MCalleesListFragment extends Fragment {

    private static final String LOG_TAG = "M2MCalleesListFragment";

    private static final String ARG_CONTENT_ARRAY = "content-array";
    private String[] mContentArray;

    /**
     * Mandatory empty constructor for the fragment manager to instantiate the
     * fragment (e.g. upon screen orientation changes).
     */
    public M2MCalleesListFragment() {
    }

    // TODO: Customize parameter initialization
    @SuppressWarnings("unused")
    public static M2MCalleesListFragment newInstance(String[] inContentArray) {
        M2MCalleesListFragment fragment = new M2MCalleesListFragment();
        Bundle args = new Bundle();
        args.putStringArray(ARG_CONTENT_ARRAY, inContentArray);
        fragment.setArguments(args);
        return fragment;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        if (getArguments() != null) {
            mContentArray = getArguments().getStringArray(ARG_CONTENT_ARRAY);
        }
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        //Add by Alec 2017.09.18 設定畫面為 IMMERSIVE_STICKY mode
        getActivity().getWindow().getDecorView().setSystemUiVisibility(
                View.SYSTEM_UI_FLAG_LAYOUT_STABLE
                        | View.SYSTEM_UI_FLAG_LAYOUT_HIDE_NAVIGATION
                        | View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN
                        | View.SYSTEM_UI_FLAG_HIDE_NAVIGATION
                        | View.SYSTEM_UI_FLAG_FULLSCREEN
                        | View.SYSTEM_UI_FLAG_IMMERSIVE_STICKY);
        View view = inflater.inflate(R.layout.fragment_callees_list, container, false);

        // Set the adapter
        if (view instanceof RecyclerView) {
            Context context = view.getContext();
            RecyclerView recyclerView = (RecyclerView) view;
            recyclerView.setNestedScrollingEnabled(false);
            recyclerView.setHasFixedSize(false);
            recyclerView.setLayoutManager(new LinearLayoutManager(context));
            recyclerView.setAdapter(new M2MCalleesListItemRecyclerViewAdapter(mContentArray));
        }
        return view;
    }


    @Override
    public void onAttach(Context context) {
        super.onAttach(context);
    }

    @Override
    public void onDetach() {
        super.onDetach();
    }

    /**
     */
    public class M2MCalleesListItemRecyclerViewAdapter extends RecyclerView.Adapter<M2MCalleesListItemRecyclerViewAdapter.ViewHolder> {

        private String[] mContentArray;

        public M2MCalleesListItemRecyclerViewAdapter(String[] inContentArray) {
            mContentArray = inContentArray;
        }

        @Override
        public ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
            View view = LayoutInflater.from(parent.getContext())
                    .inflate(R.layout.item_callees_list, parent, false);
            return new ViewHolder(view);
        }

        @Override
        public void onBindViewHolder(final ViewHolder holder, final int position) {
            holder.mContentView.setText(mContentArray[position]);

            holder.mView.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    Log.d(LOG_TAG, "onClick position=" + position);
                    ((CallerActivity)getActivity()).onM2MCalleesListFragmentClick(position);
                }
            });
        }

        @Override
        public int getItemCount() {
            return mContentArray.length;
        }

        public class ViewHolder extends RecyclerView.ViewHolder {
            public final View mView;
            public final TextView mContentView;

            public ViewHolder(View view) {
                super(view);
                mView = view;
                mContentView = (TextView) view.findViewById(R.id.content);
            }

            @Override
            public String toString() {
                return super.toString() + " '" + mContentView.getText() + "'";
            }
        }
    }
}
