package com.cht.tlcsl.webrtcsample;

import android.app.Activity;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.util.Log;
import android.view.View;
import android.view.Window;
import android.widget.Button;

import com.cht.tlcsl.webrtclib.M2MFragment;
import com.cht.tlcsl.webrtclib.WRLSignalService;

public class M2MMainActivity extends Activity {
    private static final String TAG = "M2MMainActivity";
    String userId;
    String userToken;
    String serverPreUrl;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        if (savedInstanceState == null) {
            Log.d(TAG, "savedInstanceState = null");
            Bundle extras = getIntent().getExtras();
            if(extras == null) {
                Log.d(TAG, "extras = null");
                userId = null;
                userToken = null;
                serverPreUrl = null;
            } else {
                Log.d(TAG, "extras != null");
                userId = extras.getString("userId");
                Log.d(TAG, "userId=" + userId);
                userToken = extras.getString("userToken");
                Log.d(TAG, "userToken=" + userToken);
                serverPreUrl = extras.getString("serverPreUrl");
                Log.d(TAG, "serverPreUrl=" + serverPreUrl);
            }
        } else {
            Log.d(TAG, "savedInstanceState != null");
            userId= (String) savedInstanceState.getSerializable("userID");
            userToken= (String) savedInstanceState.getSerializable("userToken");
            serverPreUrl= (String) savedInstanceState.getSerializable("serverPreUrl");
        }
        if (userId == null){
            Intent intent = new Intent();
            intent.setClass(this, LoginActivity.class);
            this.startActivity(intent);
            finish();
            return;
        }
        SharedPreferences sharedPreferences = getSharedPreferences("com.cht.tlcsl.webrtcsample.preferences", MODE_PRIVATE);
        SharedPreferences.Editor editor = sharedPreferences.edit();
        //editor.putString("AsstARToken", userToken);
        editor.putString("AsstARUserId", userId);
        editor.putString("AsstARServerUrl", serverPreUrl);
        editor.commit();

        requestWindowFeature(Window.FEATURE_NO_TITLE);
        setContentView(R.layout.activity_m2mmain);
        Button calllerButton = (Button)findViewById(R.id.callerButton);
        calllerButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                Intent intent = new Intent();
                Log.d(TAG, "userId=" + userId);
                intent.putExtra("userId", userId);
                Log.d(TAG, "userToken=" + userToken);
                intent.putExtra("userToken", userToken);
                Log.d(TAG, "serverPreURL=" + serverPreUrl);
                intent.putExtra("serverPreUrl", serverPreUrl);
                intent.setClass(M2MMainActivity.this, CallerActivity.class);
                startActivity(intent);
            }
        });
        Button lcalllerButton = (Button)findViewById(R.id.lcallerButton);
        lcalllerButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                Intent intent = new Intent();
                Log.d(TAG, "userId=" + userId);
                intent.putExtra("userId", userId);
                Log.d(TAG, "userToken=" + userToken);
                intent.putExtra("userToken", userToken);
                Log.d(TAG, "serverPreURL=" + serverPreUrl);
                intent.putExtra("serverPreUrl", serverPreUrl);
                intent.setClass(M2MMainActivity.this, LCallerActivity.class);
                startActivity(intent);
            }
        });
        Button stopButton = (Button)findViewById(R.id.stopButton);
        stopButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                M2MFragment.stopWRLSignalService(getApplicationContext());
            }
        });
        Button startButton = (Button)findViewById(R.id.startButton);
        startButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                SharedPreferences sharedPreferences = getSharedPreferences("com.cht.tlcsl.webrtcsample.preferences", MODE_PRIVATE);
                SharedPreferences.Editor editor = sharedPreferences.edit();
                editor.putString("AsstARToken", userToken);
                editor.commit();
                M2MFragment.stopWRLSignalService(getApplicationContext());
                Intent serviceIntent = new Intent(M2MMainActivity.this, WRLSignalService.class);
                serviceIntent.putExtra("UserId", userId);
                serviceIntent.putExtra("PreferencesName", "com.cht.tlcsl.webrtcsample.preferences");
                serviceIntent.putExtra("TokenName", "AsstARToken");
                serviceIntent.putExtra("LoginActionName", "com.cht.tlcsl.webrtcsample.LoginActivity");
                serviceIntent.putExtra("CalleeActionName", "com.cht.tlcsl.webrtcsample.PreCalleeActivity");
                serviceIntent.putExtra("ServerUrl", serverPreUrl);
                serviceIntent.putExtra("ConnectIcon", R.drawable.start);
                serviceIntent.putExtra("DisconnectIcon", R.drawable.stop);
                startService(serviceIntent);
            }
        });
    }

}
