package com.cht.tlcsl.webrtcsample;

import android.app.Activity;
import android.app.FragmentManager;
import android.app.FragmentTransaction;
import android.app.ProgressDialog;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.DisplayMetrics;
import android.util.Log;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;
import com.cht.tlcsl.dbl.DBLMessageListener;
import com.cht.tlcsl.dbl.DBLSurfaceViewRendererInterface;
import com.cht.tlcsl.dbl.DrawingFragment;
import com.cht.tlcsl.webrtclib.CallFragment;
import com.cht.tlcsl.webrtclib.WRLMessageListener;
import org.json.JSONArray;
import org.json.JSONObject;
import java.io.File;


public class MainActivity extends Activity implements WRLMessageListener, DBLMessageListener {
    private static final String TAG = "MainActivity";
    boolean mSignalServiceStarted;
    CallFragment mCallFragment = null;
    String userId = "cht_007";
    String userToken = "123456";
    String serverPreUrl = "https://asstar-qa.hinet.net/AsstARWeb/";
    String mCalleeSeq = "";
    int mDisplayWidthInt;
    int mDisplayHeightInt;
    private ProgressDialog mProgressDialog = null;
    DrawingFragment mDrawingFragment;
    CalleesListFragment mCalleesListFragment;
    JSONArray mCalleesListJSONArray;
    String mDisplayMessageString;
    String mErrorMessageString;
    String[] mCalleeNameArray;
    String[] mCalleeSeqArray;
    private ProgressDialog mProgressDlg = null;
    protected static final int FINSH_CHANGE_FRAGMENT_PROGRESS_THREAD = 6808;
    private Handler mHandler = new Handler() {
        @Override
        public void handleMessage(Message message) {
            switch (message.what) {
                case FINSH_CHANGE_FRAGMENT_PROGRESS_THREAD:
                    if (mProgressDialog != null) {
                        mProgressDialog.dismiss();
                        mProgressDialog = null;
                    }
                    break;
            }
        }
    };
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        if (savedInstanceState == null) {
            Log.d(TAG, "savedInstanceState = null");
            Bundle extras = getIntent().getExtras();
            if(extras == null) {
                Log.d(TAG, "extras = null");
                userId = null;
                userToken = null;
                serverPreUrl = null;
            } else {
                Log.d(TAG, "extras != null");
                userId = extras.getString("userId");
                Log.d(TAG, "userId=" + userId);
                userToken = extras.getString("userToken");
                Log.d(TAG, "userToken=" + userToken);
                serverPreUrl = extras.getString("serverPreUrl");
                Log.d(TAG, "serverPreUrl=" + serverPreUrl);
            }
        } else {
            Log.d(TAG, "savedInstanceState != null");
            userId= (String) savedInstanceState.getSerializable("userID");
            userToken= (String) savedInstanceState.getSerializable("userToken");
            serverPreUrl= (String) savedInstanceState.getSerializable("serverPreUrl");
        }        // remove title
        //userId = "cht_007";
        //userToken = "123456";
        if (userId == null){
            Intent intent = new Intent();
            intent.setClass(this, LoginActivity.class);
            this.startActivity(intent);
            finish();
            return;
        }
        requestWindowFeature(Window.FEATURE_NO_TITLE);
        getWindow().setFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN,
                WindowManager.LayoutParams.FLAG_FULLSCREEN);
        setContentView(R.layout.activity_main);
        DisplayMetrics displayMetrics = new DisplayMetrics();
        getWindowManager().getDefaultDisplay().getMetrics(displayMetrics);
        mDisplayWidthInt = displayMetrics.widthPixels;
        mDisplayHeightInt = displayMetrics.heightPixels;
        int videoHeightInt = (int) (mDisplayWidthInt * 1280 / 720);
        if (mDisplayHeightInt > videoHeightInt)
            mDisplayHeightInt = videoHeightInt;
        //加入fragment
        mCallFragment = CallFragment.newInstance(userId, userToken, serverPreUrl);
        //設定是否顯示Debug訊息
        //設定Local Video Size
        //setLocalVideoSize(int inWidth, int inHeight, boolean isPortrait);
        //預設值為 720 * 1280 直式
        //BT-200  isProtrait設false為橫式
        //mCallFragment.setLocalVideoSize(1280, 720, false);
        //設定雙向視訊建立連線之timeout,預設值為 10000 , 10秒
        mCallFragment.setWebrtcConnectTimeout(10000);
        //設定兩次retry之間的sleep time,預設值為 5000 , 5秒
        mCallFragment.setRetrySleepTime(5000);
        mCallFragment.setDebug(true);
        //mCallFragment.setDebug(false);
        getFragmentManager().beginTransaction()
                .replace(R.id.WRLCallFragment, mCallFragment)
                .commit();
        //按Call時，呼叫CallFragment的Call Method
        final Button callButton = (Button) findViewById(R.id.callButton);
        callButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                //mProgressDialog = ProgressDialog.show(MainActivity.this, "", "請稍候...", true, false);
                //隱藏支援端畫板畫面
                ImageView boradImageView = (ImageView) findViewById(R.id.boardImageView);
                boradImageView.setVisibility(View.GONE);
                //隱藏凍結畫面
                ImageView freezeImageView = (ImageView) findViewById(R.id.freezeImageView);
                freezeImageView.setVisibility(View.GONE);
                //撥打
                mCallFragment.call(R.id.local_video_view, 3);
                //Thread callThread = new Thread(new Runnable() {
                //    public void run() {
                //撥打
                //         mCallFragment.call(R.id.local_video_view);
                //     }
                // });
                // callThread.start();
            }
        });
        //按Hangup時，呼叫CallFragment的hangup Method
        Button hangupButton = (Button) findViewById(R.id.hangupButton);
        hangupButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {

                if (mDrawingFragment != null){
                    Log.d(TAG, "remove(mDrawingFragment)");
                    //將畫板Fragment從Activity刪除
                    try {
                        FragmentManager fragmentManager = getFragmentManager();
                        FragmentTransaction fragmentTransaction = fragmentManager.beginTransaction();
                        fragmentTransaction.remove(mDrawingFragment);
                        fragmentTransaction.commit();
                        mDrawingFragment = null;
                    } catch (Exception e){
                        Log.e(TAG, "remove(mDrawingFragment) exception=" + e.toString());
                        mDrawingFragment = null;
                    }
                }
                //mProgressDialog = ProgressDialog.show(MainActivity.this, "", "請稍候...", true, false);
                //掛斷
                mCallFragment.hangup();
            }
        });
        //按LED時，呼叫CallFragment的onOffFlashLight Method
        Button ledButton = (Button) findViewById(R.id.ledButton);
        ledButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                if (mCallFragment != null)
                    mCallFragment.onOffFlashLight();
            }
        });
        Button listButton = (Button) findViewById(R.id.listButton);
        listButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                //隱藏支援端畫板畫面
                ImageView boradImageView = (ImageView) findViewById(R.id.boardImageView);
                boradImageView.setVisibility(View.GONE);
                //隱藏凍結畫面
                ImageView freezeImageView = (ImageView) findViewById(R.id.freezeImageView);
                freezeImageView.setVisibility(View.GONE);
                showProgress(true, "請稍候......");
                Thread getCalleesListThread = new Thread(new Runnable() {
                    public void run() {
                        try {
                            JSONObject resultJSONObject = CallFragment.getCalleesList(userId, userToken, serverPreUrl);
                            showProgress(false, "");
                            //status_code:1000 成功
                            //status_code:2000 失敗，id不存在
                            //status_code:3000 失敗，查無有效chat room
                            //status_code:4000 失敗，系統錯誤
                            //status_code=103 Failed to connect ARWORK API
                            //status_code=104 Failed to parse ARWORK API JSON
                            //status_code=106 失敗,請勿使用UI Thread呼叫!
                            if (resultJSONObject.getInt("status_code") != 1000){
                                mErrorMessageString = "訊息 status_code=" + resultJSONObject.getInt("status_code") + ";status_msg=" + resultJSONObject.getString("status_msg");
                                runOnUiThread(new Runnable() {
                                    @Override
                                    public void run() {
                                        Toast.makeText(MainActivity.this, mErrorMessageString, Toast.LENGTH_SHORT).show();
                                    }
                                });
                            } else {
                                mCalleesListJSONArray = resultJSONObject.getJSONArray("callee");
                                mCalleeNameArray = new String[mCalleesListJSONArray.length()];
                                mCalleeSeqArray = new String[mCalleesListJSONArray.length()];
                                for (int i = 0; i < mCalleesListJSONArray.length(); i++) {
                                    mCalleeNameArray[i] = ((JSONObject) mCalleesListJSONArray.get(i)).getString("callee_name");
                                    mCalleeSeqArray[i] = ((JSONObject) mCalleesListJSONArray.get(i)).getString("callee_seq");
                                }
                                runOnUiThread(new Runnable() {
                                    @Override
                                    public void run() {
                                        mCalleesListFragment = CalleesListFragment.newInstance(mCalleeNameArray);
                                        FragmentManager fragmentManager = getFragmentManager();
                                        FragmentTransaction fragmentTransaction = fragmentManager.beginTransaction();
                                        fragmentTransaction.add(android.R.id.content, mCalleesListFragment);
                                        fragmentTransaction.commit();
                                    }
                                });

                            }
                        } catch (Exception e){
                            mErrorMessageString = "取客服清單發生錯誤！exception=" + e.toString();
                            runOnUiThread(new Runnable() {
                                @Override
                                public void run() {
                                    Toast.makeText(MainActivity.this, mErrorMessageString, Toast.LENGTH_SHORT).show();
                                }
                            });
                        }
                    }
                });
                getCalleesListThread.start();
            }
        });
    }

    @Override
    public void onReceiveWRLMessage(int MessageType, String MessageString) {
        //MessageType=1 MessageString為 Text Message
        //MessageType=2 為白板image file，MessageString為白板image file所在位置
        //MessageType=3 為影音檔，MessageString為影音檔所在位置
        //MessageType=4 為凍結image file，MessageString為凍結image file所在位置
        //MessageType=5 為白板清除Message，MessageString固定為CLEAR BOARD
        //MessageType=6 為解凍Message，MessageString固定為UNFREEZE
        //MessageType=7 為其它檔，MessageString為其它檔所在位置
        Log.d(TAG, "onReceiveWRLMessage() type=" + MessageType + ";message=" + MessageString);
        mDisplayMessageString = "訊息 type=" + MessageType + ";message=" + MessageString;
        this.runOnUiThread(new Runnable() {
            @Override
            public void run() {
                Toast.makeText(MainActivity.this, mDisplayMessageString, Toast.LENGTH_SHORT).show();
            }
        });
        switch (MessageType){
            case 1:
                break;
            case 2:
                File boardImageFile = new  File(MessageString);
                Bitmap boardBitmap = BitmapFactory.decodeFile(boardImageFile.getAbsolutePath());
                ImageView boradImageView = (ImageView) findViewById(R.id.boardImageView);
                //Matrix matrix = new Matrix();
                //matrix.setScale((float)mDisplayWidthInt/(float)boardBitmap.getWidth(), (float)mDisplayWidthInt/(float)boardBitmap.getWidth(), 0, 0);
                //boradImageView.setImageMatrix(matrix);
                boradImageView.setVisibility(View.VISIBLE);
                boradImageView.setImageBitmap(boardBitmap);
                break;
            case 3:
                break;
            case 4:
                File freezeImageFile = new File(MessageString);
                Bitmap freezeBitmap = BitmapFactory.decodeFile(freezeImageFile.getAbsolutePath());
                ImageView freezeImageView = (ImageView) findViewById(R.id.freezeImageView);
                //Matrix freezeMatrix = new Matrix();
                //freezeMatrix.setScale((float)mDisplayWidthInt/(float)freezeBitmap.getWidth(), (float)mDisplayWidthInt/(float)freezeBitmap.getWidth(), 0, 0);
                //freezeImageView.setImageMatrix(freezeMatrix);
                freezeImageView.setVisibility(View.VISIBLE);
                freezeImageView.setImageBitmap(freezeBitmap);
                if (mDrawingFragment != null)
                    mDrawingFragment.setFreeze();
                break;
            case 5:
                boradImageView = (ImageView) findViewById(R.id.boardImageView);
                boradImageView.setVisibility(View.GONE);
                if (mDrawingFragment != null)
                    mDrawingFragment.clearDrawingBoard();
                break;
            case 6:
                freezeImageView = (ImageView) findViewById(R.id.freezeImageView);
                freezeImageView.setVisibility(View.GONE);
                if (mDrawingFragment != null)
                    mDrawingFragment.setUnfreeze();
                break;
            case 7:
            default:
                break;
        }
    }

    @Override
    public void onConnect(String inCalleeSeq) {
        //例如可在此改變Call Button由不可按變為可掛斷
        //mHandler.sendEmptyMessage(FINSH_CHANGE_FRAGMENT_PROGRESS_THREAD);
        Log.d(TAG, "onConnect() inCalleeSeq = " + inCalleeSeq);
        mCalleeSeq = inCalleeSeq;
        this.runOnUiThread(new Runnable() {
            @Override
            public void run() {
                Toast.makeText(MainActivity.this, "onConnect()", Toast.LENGTH_SHORT).show();
                //隱藏支援端畫板畫面
                ImageView boradImageView = (ImageView) findViewById(R.id.boardImageView);
                boradImageView.setVisibility(View.GONE);
                //隱藏凍結畫面
                ImageView freezeImageView = (ImageView) findViewById(R.id.freezeImageView);
                freezeImageView.setVisibility(View.GONE);
                //顯示客服編號
                TextView calleeTextView = (TextView) findViewById(R.id.calleeTextView);
                calleeTextView.setText("專家:" + mCalleeSeq);

                //將畫板Fragment加入Activity
                FragmentManager fragmentManager = getFragmentManager();
                FragmentTransaction fragmentTransaction = fragmentManager.beginTransaction();
                mDrawingFragment = DrawingFragment.newInstance();
                fragmentTransaction.replace(R.id.drawingFrameLayout, mDrawingFragment, "DrawingFragment");
                fragmentTransaction.commit();
                mDrawingFragment.setDBLSurfaceViewRendererInterface((DBLSurfaceViewRendererInterface) findViewById(R.id.local_video_view));
            }
        });
    }

    @Override
    public void onDisconnect() {
        //例如可在此改變Call Button變為可撥打
        //mHandler.sendEmptyMessage(FINSH_CHANGE_FRAGMENT_PROGRESS_THREAD);
        Log.d(TAG, "onDisconnect()");
        this.runOnUiThread(new Runnable() {
            @Override
            public void run() {
                //隱藏支援端畫板畫面
                ImageView boradImageView = (ImageView) findViewById(R.id.boardImageView);
                boradImageView.setVisibility(View.GONE);
                //隱藏凍結畫面
                ImageView freezeImageView = (ImageView) findViewById(R.id.freezeImageView);
                freezeImageView.setVisibility(View.GONE);
                //FragmentManager fragmentManager = getFragmentManager();
                //DrawingFragment drawingFragment = (DrawingFragment) fragmentManager.findFragmentByTag("DrawingFragment");
                if (mDrawingFragment != null){
                    Log.d(TAG, "remove(mDrawingFragment)");
                    //將畫板Fragment從Activity刪除
                    try {
                        FragmentManager fragmentManager = getFragmentManager();
                        FragmentTransaction fragmentTransaction = fragmentManager.beginTransaction();
                        fragmentTransaction.remove(mDrawingFragment);
                        fragmentTransaction.commit();
                        mDrawingFragment = null;
                    } catch (Exception e){
                        Log.e(TAG, "remove(mDrawingFragment) exception=" + e.toString());
                        mDrawingFragment = null;
                    }
                }
                Toast.makeText(MainActivity.this, "onDisconnect()", Toast.LENGTH_SHORT).show();
            }
        });
        //finish();
    }

    @Override
    public void onError(int errorType, String descriptionString) {
        //ErrorType=11 Failed to create peer connection
        //ErrorType=12 Failed to open camera
        //ErrorType=13 Failed to add stream
        //ErrorType=14 Failed to create SDP
        //ErrorType=15 Failed to create ice connection
        //ErrorType=16 Failed to set SDP
        //ErrorType=17 Failed to initializeAndroidGlobals
        //ErrorType=21 Failed to connect WebSocket
        //ErrorType=22 Failed to register WebSocket
        //ErrorType=23 Failed to send WebSocket
        //ErrorType=24 Failed to send PostServer
        //ErrorType=25 無法連接訊息伺服器
        //ErrorType=26 與遠端失去連線
        //ErrorType=31 Failed to parse RoomParameters
        //ErrorType=32 onWebSocketError
        //ErrorType=33 not initiator receive answer
        //ErrorType=34  initiator receive offer
        //ErrorType=35 Unexpected WebSocket message
        //ErrorType=36 WebSocket message JSON parsing error
        //ErrorType=37 WebSocket non connected
        //ErrorType=38 WebSocket send error
        //ErrorType=39 Loopback room is busy
        //ErrorType=101 Permissions not granted
        //ErrorType=103 Failed to connect ARWORK API
        //ErrorType=104 Failed to parse ARWORK API JSON
        //ErrorType=105 使用者取消連線
        //mHandler.sendEmptyMessage(FINSH_CHANGE_FRAGMENT_PROGRESS_THREAD);
        Log.d(TAG, "onError() type=" + errorType + ";description=" + descriptionString);
        mErrorMessageString = "訊息 type=" + errorType + ";description=" + descriptionString;
        this.runOnUiThread(new Runnable() {
            @Override
            public void run() {
                //隱藏支援端畫板畫面
                ImageView boradImageView = (ImageView) findViewById(R.id.boardImageView);
                boradImageView.setVisibility(View.GONE);
                //隱藏凍結畫面
                ImageView freezeImageView = (ImageView) findViewById(R.id.freezeImageView);
                freezeImageView.setVisibility(View.GONE);
                //FragmentManager fragmentManager = getFragmentManager();
                //DrawingFragment drawingFragment = (DrawingFragment) fragmentManager.findFragmentByTag("DrawingFragment");
                if (mDrawingFragment != null){
                    Log.d(TAG, "remove(mDrawingFragment)");
                    //將畫板Fragment從Activity刪除
                    try {
                        FragmentManager fragmentManager = getFragmentManager();
                        FragmentTransaction fragmentTransaction = fragmentManager.beginTransaction();
                        fragmentTransaction.remove(mDrawingFragment);
                        fragmentTransaction.commit();
                        mDrawingFragment = null;
                    } catch (Exception e){
                        Log.e(TAG, "remove(mDrawingFragment) exception=" + e.toString());
                        mDrawingFragment = null;
                    }
                }
                Toast.makeText(MainActivity.this, mErrorMessageString, Toast.LENGTH_SHORT).show();
            }
        });
        //finish();
    }
    //手機端畫板畫面已儲存完畢之Callback
    @Override
    public void onDBLFileSaved(String filePathString){
        Log.d(TAG, "onDBLFileSaved()");
        //傳送畫板畫面檔案至支援端
        if (mCallFragment != null)
            mCallFragment.sendLocalBoardFile(filePathString);
        //Toast.makeText(this, "onDBLFileSaved():" + filePathString, Toast.LENGTH_SHORT).show();
    }
    //手機端使用者按下清除畫板之Callback
    @Override
    public void onDBLClear(){
        Log.d(TAG, "onDBLClear()");
        //清除支援端畫板
        ImageView boradImageView = (ImageView) findViewById(R.id.boardImageView);
        boradImageView = (ImageView) findViewById(R.id.boardImageView);
        boradImageView.setVisibility(View.GONE);
        //傳送清除畫板message至支援端
        if (mCallFragment != null)
            mCallFragment.sendClearingBoardRequest();
        //Toast.makeText(this, "onDBLClear()", Toast.LENGTH_SHORT).show();
    }
    //手機端凍結畫面已儲存完畢之Callback
    @Override
    public void onDBLFreezeFileSaved(String filePathString){
        Log.d(TAG, "onDBLFreezeFileSaved()");
        //顯示手機端凍結畫面
        File freezeImageFile = new File(filePathString);
        Bitmap freezeBitmap = BitmapFactory.decodeFile(freezeImageFile.getAbsolutePath());
        ImageView freezeImageView = (ImageView) findViewById(R.id.freezeImageView);
        freezeImageView.setVisibility(View.VISIBLE);
        freezeImageView.setImageBitmap(freezeBitmap);
        //傳送凍結畫面檔案至支援端
        if (mCallFragment != null)
            mCallFragment.sendLocalFreezingFile(filePathString);
        //Toast.makeText(MainActivity.this, "onDBLFreezeFileSaved()", Toast.LENGTH_SHORT).show();
    }
    //手機端使用者按下解凍按鈕之Callback
    @Override
    public void onDBLUnfreeze(){
        Log.d(TAG, "onDBLUnfreeze()");
        //清除手機端凍結畫面
        ImageView freezeImageView = (ImageView) findViewById(R.id.freezeImageView);
        freezeImageView.setVisibility(View.GONE);
        //傳送解凍message至支援端
        if (mCallFragment != null)
            mCallFragment.sendUnfreezingBoardRequest();
        //Toast.makeText(MainActivity.this, "onDBLUnfreeze()", Toast.LENGTH_SHORT).show();
    }
    //手機端使用者按下文字輸入按鈕之Callback
    int mInputCount = 0;
    @Override
    public void onDBLInputButtonPressed(){
        Log.d(TAG, "onDBLInputButtonPressed()");
        //在此顯示使用者文字輸入畫面，輸入完成以下方式傳給對方
    }
    @Override
    //手機端使用者按下傳送檔案按鈕之Callback
    public void onDBLFileButtonPressed(){
        Log.d(TAG, "onDBLFileButtonPressed()");
        //在此顯示使用者選取檔案畫面，輸入完成以下方式傳給對方
    }
    @Override
    //手機端使用者按下LED按鈕之Callback
    public void onDBLLedButtonPressed() {
        Log.d(TAG, "onDBLLedButtonPressed()");
        //按LED時，呼叫CallFragment的onOffFlashLight Method
        if (mCallFragment != null)
            mCallFragment.onOffFlashLight();
    }
    public void onCalleesListFragmentClick(int inItemNum){
        Log.d(TAG, "onCalleesListFragmentClick() inItemNum=" + inItemNum);
        if (mCalleesListFragment != null){
            Log.d(TAG, "remove(mCalleesListFragment)");
            try {
                FragmentManager fragmentManager = getFragmentManager();
                FragmentTransaction fragmentTransaction = fragmentManager.beginTransaction();
                fragmentTransaction.remove(mCalleesListFragment);
                fragmentTransaction.commit();
                mCalleesListFragment = null;
                mCallFragment.call(mCalleeSeqArray[inItemNum], R.id.local_video_view, 3);
            } catch (Exception e){
                Log.e(TAG, "remove(mCalleesListFragment) exception=" + e.toString());
                mCalleesListFragment = null;
            }
        }
    }
    private void showProgress(final boolean show, String message) {

        if ( show ) {
            if (mProgressDlg != null) {
                mProgressDlg.dismiss();
                mProgressDlg = null;
            }
            mProgressDlg = ProgressDialog.show(this, null, message, true, false);

        } else {
            if (mProgressDlg != null) {
                mProgressDlg.dismiss();
                mProgressDlg = null;
            }

        }

    }
    //@Override
    //public void onBackPressed() {
        // do nothing.
    //}
    @Override
    public void onResume() {
        super.onResume();
        Log.d(TAG, "onResume()");
    }
    @Override
    public void onPause() {
        super.onPause();
        Log.d(TAG, "onPause()");
    }
}
