package com.cht.tlcsl.webrtcsample;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.media.Ringtone;
import android.media.RingtoneManager;
import android.os.Bundle;
import android.util.Log;
import android.view.WindowManager;
import android.widget.Toast;

import com.cht.tlcsl.webrtclib.M2MFragment;
import com.cht.tlcsl.webrtclib.WRLSignalAgent;
import com.cht.tlcsl.webrtclib.WRLHangUpListener;

import org.json.JSONObject;

import static android.R.style.Theme_DeviceDefault_Light_Dialog_Alert;

public class PreCalleeActivity extends Activity {
    private static final String TAG = "PreCalleeActivity";
    String mCallerId;
    String mCallerName;
    String mScreenType;
    String userId;
    String userToken;
    String serverPreUrl;
    private ProgressDialog mProgressDialog = null;
    String mDisplayMessageString;
    String mErrorMessageString;
    private Ringtone mRingtone;
    private AlertDialog mAlertDialog;
    private WRLSignalAgent mWRLSignalAgent;
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        if (savedInstanceState == null) {
            Log.d(TAG, "savedInstanceState = null");
            Bundle extras = getIntent().getExtras();
            if(extras == null) {
                Log.d(TAG, "extras = null");
                mCallerId = null;
                mCallerName = null;
            } else {
                Log.d(TAG, "extras != null");
                mCallerId = extras.getString("CallerId");
                Log.d(TAG, "mCallerId=" + mCallerId);
                mCallerName = extras.getString("CallerName");
                Log.d(TAG, "mCallerName=" + mCallerName);
                mScreenType = extras.getString("ScreenType");
                Log.d(TAG, "mScreenType=" + mScreenType);
            }
        } else {
            Log.d(TAG, "savedInstanceState != null");
            mCallerId= (String) savedInstanceState.getSerializable("CallerId");
            mCallerName= (String) savedInstanceState.getSerializable("CallerName");
        }
        if (mCallerId == null){
            Intent intent = new Intent();
            intent.setClass(this, M2MMainActivity.class);
            this.startActivity(intent);
            finish();
            return;
        }
        SharedPreferences sharedPreferences = getSharedPreferences("com.cht.tlcsl.webrtcsample.preferences", MODE_PRIVATE);
        userId = sharedPreferences.getString("AsstARUserId", null);
        userToken = sharedPreferences.getString("AsstARToken", null);
        serverPreUrl = sharedPreferences.getString("AsstARServerUrl", null);
        if (userId == null){
            Intent intent = new Intent();
            intent.setClass(this, LoginActivity.class);
            this.startActivity(intent);
            finish();
            return;
        }

        setContentView(R.layout.activity_callee);
        mWRLSignalAgent = new WRLSignalAgent();
        mWRLSignalAgent.setWRLHangUpListener(this, new HangUpListener());
        getWindow().addFlags(WindowManager.LayoutParams.FLAG_SHOW_WHEN_LOCKED | WindowManager.LayoutParams.FLAG_DISMISS_KEYGUARD | WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON | WindowManager.LayoutParams.FLAG_TURN_SCREEN_ON | WindowManager.LayoutParams.FLAG_ALLOW_LOCK_WHILE_SCREEN_ON);
    }
    private class HangUpListener implements WRLHangUpListener {
        @Override
        public void onRemoteHangUp(String RemoteId) {
            Log.d(TAG, "onRemoteHangUp() RemoteId=" + RemoteId);
            if (RemoteId != null && RemoteId.equalsIgnoreCase(mCallerId)){
                if (mRingtone != null){
                    mRingtone.stop();
                    mRingtone = null;
                }
                if (mAlertDialog != null){
                    mAlertDialog.dismiss();
                    mAlertDialog = null;
                }
                Toast.makeText(PreCalleeActivity.this, "對方取消", Toast.LENGTH_SHORT).show();
                finish();
            }
        }
    }
    private void showProgress(final boolean show, String message) {
        if ( show ) {
            if (mProgressDialog != null) {
                mProgressDialog.dismiss();
                mProgressDialog = null;
            }
            mProgressDialog = ProgressDialog.show(this, null, message, true, false);
        } else {
            if (mProgressDialog != null) {
                mProgressDialog.dismiss();
                mProgressDialog = null;
            }
        }
    }
    //@Override
    //public void onBackPressed() {
        // do nothing.
    //}
    @Override
    public void onResume() {
        super.onResume();
        Log.d(TAG, "onResume()");
        if (mRingtone == null){
            mRingtone = RingtoneManager.getRingtone(this, RingtoneManager.getDefaultUri(RingtoneManager.TYPE_RINGTONE));
            mRingtone.play();
        }
        if (mAlertDialog == null){
            AlertDialog.Builder builder = new AlertDialog.Builder(PreCalleeActivity.this, Theme_DeviceDefault_Light_Dialog_Alert);
            builder.setCancelable(false);
            mAlertDialog = builder.setTitle("訊息").setMessage(mCallerName + "來電，是否接聽？").setPositiveButton("是", dialogClickListener).setNegativeButton("否", dialogClickListener).create();
            mAlertDialog.show();
        }
    }
    @Override
    public void onPause() {
        super.onPause();
        Log.d(TAG, "onPause()");
    }
    DialogInterface.OnClickListener dialogClickListener = new DialogInterface.OnClickListener() {
        @Override
        public void onClick(DialogInterface dialog, int which) {
            if (mRingtone != null){
                mRingtone.stop();
                mRingtone = null;
            }
            if (mAlertDialog != null){
                mAlertDialog.dismiss();
                mAlertDialog = null;
            }
            switch (which){
                case DialogInterface.BUTTON_POSITIVE:
                    //Yes button clicked
                    //mM2MFragment.callee(mCallerId, R.id.local_video_view, R.id.remote_video_view, true);
                    Intent intent = new Intent();
                    intent.putExtra("CallerId", mCallerId);
                    intent.putExtra("CallerName", mCallerName);
                    if (mScreenType.equalsIgnoreCase("P")){
                        //Caller為直式螢幕
                        intent.setClass(PreCalleeActivity.this, CalleeActivity.class);
                    }
                    else{
                        //Caller為横式螢幕
                        intent.setClass(PreCalleeActivity.this, LCalleeActivity.class);
                    }
                    startActivity(intent);
                    finish();
                    break;
                case DialogInterface.BUTTON_NEGATIVE:
                    //No button clicked
                    //mM2MFragment.callee(mCallerId, R.id.local_video_view, R.id.remote_video_view, false);
                    showProgress(true, "請稍候......");
                    Thread callThread = new Thread(new Runnable() {
                        public void run() {
                            try {
                                JSONObject resultJSONObject = M2MFragment.calleeAnswerNo(userId, userToken, serverPreUrl, mCallerId);
                                showProgress(false, "");
                                //status_code:1000 成功
                                //status_code:2000 失敗，id不存在
                                //status_code:3000 失敗，查無有效chat room
                                //status_code:4000 失敗，系統錯誤
                                //status_code=103 Failed to connect ARWORK API
                                //status_code=104 Failed to parse ARWORK API JSON
                                //status_code=106 失敗,請勿使用UI Thread呼叫!
                                Log.d(TAG, "M2MFragment.calleeAnswerNo() result=" + resultJSONObject.toString());
                                if (resultJSONObject.getInt("status_code") != 1000){
                                    mErrorMessageString = "訊息 status_code=" + resultJSONObject.getInt("status_code") + ";status_msg=" + resultJSONObject.getString("status_msg");
                                    runOnUiThread(new Runnable() {
                                        @Override
                                        public void run() {
                                            Toast.makeText(PreCalleeActivity.this, mErrorMessageString, Toast.LENGTH_SHORT).show();
                                            finish();
                                        }
                                    });
                                } else {
                                    runOnUiThread(new Runnable() {
                                        @Override
                                        public void run() {
                                            Toast.makeText(PreCalleeActivity.this, "回覆無法接聽成功", Toast.LENGTH_SHORT).show();
                                            finish();
                                        }
                                    });
                                }
                            } catch (Exception e){
                                mErrorMessageString = "回覆無法接聽發生錯誤！exception=" + e.toString();
                                runOnUiThread(new Runnable() {
                                    @Override
                                    public void run() {
                                        Toast.makeText(PreCalleeActivity.this, mErrorMessageString, Toast.LENGTH_SHORT).show();
                                        finish();
                                    }
                                });
                            }
                        }
                    });
                    callThread.start();
                    break;
            }
        }
    };

    @Override
    protected void onDestroy() {
        Log.d(TAG, "onDestroy()");
        if (mWRLSignalAgent != null)
            mWRLSignalAgent.removeWRLHangUpListener(this);
        super.onDestroy();
    }
}
